@@ -88,7 +88,6 @@
 
 #else  /* !NATIVE_WIN32 */
 
-#ifndef __EMX__
 /* Unix */
 
 #define G_DIR_SEPARATOR '/'
@@ -96,16 +95,6 @@
 #define G_SEARCHPATH_SEPARATOR ':'
 #define G_SEARCHPATH_SEPARATOR_S ":"
 
-#else
-/* EMX/OS2 */
-
-#define G_DIR_SEPARATOR '/'
-#define G_DIR_SEPARATOR_S "/"
-#define G_SEARCHPATH_SEPARATOR ';'
-#define G_SEARCHPATH_SEPARATOR_S ";"
-
-#endif
-
 #endif /* !NATIVE_WIN32 */
 
 #ifdef __cplusplus
@@ -713,7 +702,6 @@ typedef struct _GTuples		GTuples;
 typedef union  _GTokenValue	GTokenValue;
 typedef struct _GIOChannel	GIOChannel;
 
-/* Tree traverse flags */
 typedef enum
 {
   G_TRAVERSE_LEAFS	= 1 << 0,
@@ -722,7 +710,6 @@ typedef enum
   G_TRAVERSE_MASK	= 0x03
 } GTraverseFlags;
 
-/* Tree traverse orders */
 typedef enum
 {
   G_IN_ORDER,
@@ -2419,7 +2406,7 @@ struct _GSourceFuncs
 			GTimeVal *current_time,
 			gpointer  user_data);
   gboolean (*dispatch) (gpointer  source_data, 
-			GTimeVal *dispatch_time,
+			GTimeVal *current_time,
 			gpointer  user_data);
   GDestroyNotify destroy;
 };
@@ -2447,7 +2434,7 @@ gboolean g_source_remove_by_source_data      (gpointer       source_data);
 gboolean g_source_remove_by_funcs_user_data  (GSourceFuncs  *funcs,
 					      gpointer       user_data);
 
-void g_get_current_time		        (GTimeVal	*result);
+void g_get_current_time		    (GTimeVal	   *result);
 
 /* Running the main loop */
 GMainLoop*	g_main_new		(gboolean	 is_running);
